@@ -14,7 +14,7 @@
 
 #define INQUIRY_CMD     0x12
 #define INQUIRY_CMDLEN  6
-#define HEAVY_CHECK_COUNT	10
+#define HEAVY_CHECK_COUNT       10
 
 struct emc_clariion_checker_context {
 	int fd;
@@ -43,16 +43,19 @@ int emc_clariion(char *devt, char *msg, void **context)
 	 * initialize it
 	 */
 	if (!ctxt) {
-		ctxt = malloc(sizeof(struct emc_clariion_checker_context *));
-		memset(ctxt, 0, sizeof(struct emc_clariion_checker_context *));
-	}
-	if (!ctxt) {
-		MSG("cannot allocate context");
-		return -1;
+		ctxt = malloc(sizeof(struct emc_clariion_checker_context));
+		memset(ctxt, 0, sizeof(struct emc_clariion_checker_context));
+
+		if (!ctxt) {
+			MSG("cannot allocate context");
+			return -1;
+		}
+		if (context)
+			*context = ctxt;
 	}
 	ctxt->run_count++;
 
-	if (ctxt->run_count % HEAVY_CHECK_COUNT) {
+	if ((ctxt->run_count % HEAVY_CHECK_COUNT) == 0) {
 		ctxt->run_count = 0;
 		/* do stuff */
 	}
