@@ -121,7 +121,7 @@ devt2devname (char *devname, char *devt)
 	char attr_value[16];
 
 	if (sysfs_get_mnt_path(sysfs_path, FILE_NAME_SIZE)) {
-		fprintf(stderr, "-D feature available with sysfs only\n");
+		fprintf(stderr, "feature available with sysfs only\n");
 		exit(1);
 	}
 		
@@ -149,11 +149,11 @@ devt2devname (char *devname, char *devt)
 			}
 			sysfs_get_name_from_path(attr_path, devname,
 						 FILE_NAME_SIZE);
-			break;
+			return 0;
 		}
 	}
 	sysfs_close_directory(sdir);
-	return 0;
+	return 1;
 }
 
 static int
@@ -307,6 +307,19 @@ sysfs_devinfo(struct path * curpath)
 			strlen(attr_buff) - 1);
 	dbg("dev_t = %s", curpath->dev_t);
 
+	/*
+	 * size
+	 */
+	if(safe_sprintf(attr_path, "%s/block/%s/size",
+		sysfs_path, curpath->dev)) {
+		fprintf(stderr, "attr_path too small\n");
+		return 1;
+	}
+	if (0 > readattr(attr_path, attr_buff))
+		return 1;
+	curpath->size = strtoul(attr_buff, NULL, 0);
+	dbg("size = %lu", curpath->size);
+
 	/*
 	 * host / bus / target / lun
 	 */
@@ -554,6 +567,7 @@ static int fill_0x83_id(char *page_83, char *serial)
 	return 0;
 }
 
+#if 0
 unsigned long
 get_disk_size (char * devname) {
 	unsigned long size;
@@ -577,6 +591,7 @@ get_disk_size (char * devname) {
 	dbg("get_disk_size need sysfs");
 	return -1;
 }
+#endif
 
 /*
  * get EVPD page 0x83 off 8
