@@ -111,16 +111,13 @@ get_pathvec_sysfs (vector pathvec)
 	 * if devt specified on the cmd line,
 	 * only consider affiliated paths
 	 */
-	if (conf->devt) {
-		if (devt2devname(buff, conf->devt))
-			return 1;
-		
+	if (conf->dev && !devt2devname(buff, conf->dev)) {
+		dbg("limited scope = %s", conf->dev);
 		curpath = zalloc(sizeof (struct path));
 		if(safe_sprintf(curpath->dev, "%s", buff)) {
 			fprintf(stderr, "curpath->dev too small\n");
 			exit(1);
 		}
-
 		if (devinfo(curpath))
 			return 1;
 
@@ -164,6 +161,7 @@ get_pathvec_sysfs (vector pathvec)
 		}
 		if (memcmp(empty_buff, refwwid, WWID_SIZE) != 0 && 
 		    memcmp(curpath->wwid, refwwid, WWID_SIZE) != 0) {
+			dbg("skip path %s : out of scope", curpath->dev);
 			free(curpath);
 			continue;
 		}
@@ -280,6 +278,28 @@ print_mp (struct multipath * mpp)
 	struct path * pp = NULL;
 	struct pathgroup * pgp = NULL;
 
+	if (mpp->action == ACT_NOTHING)
+		return;
+
+	if (conf->verbosity > 1) {
+		switch (mpp->action) {
+		case ACT_RELOAD:
+			printf("%s: ", ACT_RELOAD_STR);
+			break;
+
+		case ACT_CREATE:
+			printf("%s: ", ACT_CREATE_STR);
+			break;
+
+		case ACT_SWITCHPG:
+			printf("%s: ", ACT_SWITCHPG_STR);
+			break;
+
+		default:
+			break;
+		}
+	}
+
 	if (mpp->alias)
 		printf("%s", mpp->alias);
 
@@ -292,8 +312,14 @@ print_mp (struct multipath * mpp)
 
 	printf("\n");
 
-	if (mpp->size)
+	if (mpp->size < 2000)
+		printf("[size=%lu kB]", mpp->size / 2);
+	else if (mpp->size < (2000 * 1024))
 		printf("[size=%lu MB]", mpp->size / 2 / 1024);
+	else if (mpp->size < (2000 * 1024 * 1024))
+		printf("[size=%lu GB]", mpp->size / 2 / 1024 / 1024);
+	else
+		printf("[size=%lu TB]", mpp->size / 2 / 1024 / 1024 / 1024);
 
 	if (mpp->features)
 		printf("[features=\"%s\"]", mpp->features);
@@ -312,23 +338,24 @@ print_mp (struct multipath * mpp)
 		if (mpp->selector)
 			printf("%s ", mpp->selector);
 
+		switch (pgp->status) {
+		case PGSTATE_ENABLED:
+			printf("[enabled]");
+			break;
+		case PGSTATE_DISABLED:
+			printf("[disabled]");
+			break;
+		case PGSTATE_ACTIVE:
+			printf("[active]");
+			break;
+		default:
+			break;
+		}
 		if (mpp->nextpg && mpp->nextpg == j + 1)
 			printf("[first]");
 
-		switch (pgp->status) {
-		case PGSTATE_ENABLED:
-			printf("[enabled]\n");
-			break;
-		case PGSTATE_DISABLED:
-			printf("[disabled]\n");
-			break;
-		case PGSTATE_ACTIVE:
-			printf("[active]\n");
-			break;
-		default:
-			printf("\n");
-			break;
-		}
+		printf("\n");
+
 		vector_foreach_slot (pgp->paths, pp, i)
 			print_path(pp, PRINT_PATH_SHORT);
 	}
@@ -348,62 +375,63 @@ print_all_mp (vector mp)
 static void
 coalesce_paths (vector mp, vector pathvec)
 {
-	int k, i, already_done;
+	int k, i;
 	char empty_buff[WWID_SIZE];
 	struct multipath * mpp;
 	struct path * pp1;
 	struct path * pp2;
 
-	already_done = 0;
-	memset (empty_buff, 0, WWID_SIZE);
+	memset(empty_buff, 0, WWID_SIZE);
 
 	vector_foreach_slot (pathvec, pp1, k) {
 		/* skip this path for some reason */
 
 		/* 1. if path has no unique id */
-		if (memcmp (empty_buff, pp1->wwid, WWID_SIZE) == 0)
+		if (memcmp(empty_buff, pp1->wwid, WWID_SIZE) == 0)
 			continue;
 
-		/* 2. if mp with this uid already instanciated */
-		vector_foreach_slot (mp, mpp, i)
-			if (0 == strcmp (mpp->wwid, pp1->wwid))
-				already_done = 1;
-
-		if (already_done) {
-			already_done = 0;
+		/* 2. if path already coalesced */
+		if (pp1->mpp)
 			continue;
-		}
 
 		/*
 		 * at this point, we know we really got a new mp
 		 */
-		vector_alloc_slot(mp);
 		mpp = zalloc(sizeof(struct multipath));
-		strcpy (mpp->wwid, pp1->wwid);
+		pp1->mpp = mpp;
+		strcpy(mpp->wwid, pp1->wwid);
+		mpp->size = pp1->size;
 
 		mpp->mpe = find_mp(pp1->wwid);
 		mpp->hwe = find_hw(pp1->vendor_id, pp1->product_id);
 
 		mpp->paths = vector_alloc();
 		vector_alloc_slot (mpp->paths);
-		vector_set_slot (mpp->paths, VECTOR_SLOT(pathvec, k));
-
-		if (mpp->size == 0 && pp1->state == PATH_UP)
-			mpp->size = get_disk_size(pp1->dev);
+		vector_set_slot (mpp->paths, pp1);
 
 		for (i = k + 1; i < VECTOR_SIZE(pathvec); i++) {
 			pp2 = VECTOR_SLOT(pathvec, i);
-			if (0 == strcmp(pp1->wwid, pp2->wwid)) {
-				vector_alloc_slot(mpp->paths);
-				vector_set_slot(mpp->paths,
-						VECTOR_SLOT(pathvec, i));
 
-				if (mpp->size == 0 && pp2->state == PATH_UP)
-					mpp->size = get_disk_size(pp2->dev);
+			if (strcmp(pp1->wwid, pp2->wwid))
+				continue;
+			
+			pp2->mpp = mpp;
 
+			if (pp2->size != mpp->size) {
+				/*
+				 * ouch, avoid feeding that to the DM
+				 */
+				dbg("path size mismatch : discard %s",
+				     mpp->wwid);
+				mpp->action = ACT_NOTHING;
 			}
+			vector_alloc_slot(mpp->paths);
+			vector_set_slot(mpp->paths, VECTOR_SLOT(pathvec, i));
+		}
+		if (mpp) {
+			vector_alloc_slot(mp);
+			vector_set_slot(mp, mpp);
 		}
-		vector_set_slot(mp, mpp);
 	}
 }
 
@@ -588,7 +616,7 @@ setup_map (struct multipath * mpp)
 	mpp->nextpg = 1;
 	vector_foreach_slot (mpp->pg, pgp, i) {
 		vector_foreach_slot (pgp->paths, pp, j) {
-			pgp->id ^= (int)pp;
+			pgp->id ^= (long)pp;
 			if (pp->state != PATH_DOWN)
 				pgp->priority += pp->priority;
 		}
@@ -606,6 +634,45 @@ setup_map (struct multipath * mpp)
 	return 0;
 }
 
+/*
+ * detect if a path is in the map we are about to create but not in the
+ * current one (triggers a valid reload)
+ * if a path is in the current map but not in the one we are about to create,
+ * don't reload : it may come back latter so save the reload burden
+ */
+static int
+pgcmp2 (struct multipath * mpp, struct multipath * cmpp)
+{
+	int i, j, k, l;
+	struct pathgroup * pgp;
+	struct pathgroup * cpgp;
+	struct path * pp;
+	struct path * cpp;
+	int found = 0;
+
+	vector_foreach_slot (mpp->pg, pgp, i) {
+		vector_foreach_slot (pgp->paths, pp, j) {
+			vector_foreach_slot (cmpp->pg, cpgp, k) {
+				vector_foreach_slot (cpgp->paths, cpp, l) {
+					if (pp == cpp) {
+						found = 1;
+						break;
+					}
+				}
+				if (found)
+					break;
+			}
+			if (found) {
+				found = 0;
+				break;
+			} else
+				return 1;
+		}
+	}
+	return 0;
+}
+
+#if 0
 static int
 pgcmp (struct multipath * mpp, struct multipath * cmpp)
 {
@@ -627,55 +694,61 @@ pgcmp (struct multipath * mpp, struct multipath * cmpp)
 	}
 	return r;
 }
+#endif
 
-static int
+static void
 select_action (struct multipath * mpp, vector curmp)
 {
 	int i;
 	struct multipath * cmpp;
 
-	/*
-	 * FIXME : ACT_RENAME, move to action |= ACT_*
-	 */
-
 	vector_foreach_slot (curmp, cmpp, i) {
 		if (strncmp(cmpp->alias, mpp->alias, strlen(mpp->alias)))
 			continue;
 
 		if (cmpp->size != mpp->size) {
 			dbg("size different than current");
-			return ACT_RELOAD;
+			mpp->action = ACT_RELOAD;
+			return;
 		}
 		if (strncmp(cmpp->features, mpp->features,
 			    strlen(mpp->features))) {
 			dbg("features different than current");
-			return ACT_RELOAD;
+			mpp->action =  ACT_RELOAD;
+			return;
 		}
 		if (strncmp(cmpp->hwhandler, mpp->hwhandler,
 			    strlen(mpp->hwhandler))) {
 			dbg("hwhandler different than current");
-			return ACT_RELOAD;
+			mpp->action = ACT_RELOAD;
+			return;
 		}
 		if (strncmp(cmpp->selector, mpp->selector,
 			    strlen(mpp->selector))) {
 			dbg("selector different than current");
-			return ACT_RELOAD;
+			mpp->action = ACT_RELOAD;
+			return;
 		}
 		if (VECTOR_SIZE(cmpp->pg) != VECTOR_SIZE(mpp->pg)) {
-			dbg("different path group topology");
-			return ACT_RELOAD;
+			dbg("different number og PG");
+			mpp->action = ACT_RELOAD;
+			return;
 		}
-		if (pgcmp(mpp, cmpp)) {
+		if (pgcmp2(mpp, cmpp)) {
 			dbg("different path group topology");
-			return ACT_RELOAD;
+			mpp->action = ACT_RELOAD;
+			return;
 		}
 		if (cmpp->nextpg != mpp->nextpg) {
 			dbg("nextpg different than current");
-			return ACT_SWITCHPG;
+			mpp->action = ACT_SWITCHPG;
+			return;
 		}
-		return ACT_NOTHING;
+		mpp->action = ACT_NOTHING;
+		return;
 	}
-	return ACT_CREATE;
+	mpp->action = ACT_CREATE;
+	return;
 }
 
 static int
@@ -686,11 +759,12 @@ reinstate_paths (struct multipath * mpp)
 	struct path * pp;
 
 	vector_foreach_slot (mpp->pg, pgp, i) {
-		if (pgp->status == PGSTATE_DISABLED ||
-		    pgp->status == PGSTATE_ACTIVE)
-			continue;
-
 		vector_foreach_slot (pgp->paths, pp, j) {
+			if (pp->state != PATH_UP &&
+			    (pgp->status == PGSTATE_DISABLED ||
+			     pgp->status == PGSTATE_ACTIVE))
+				continue;
+
 			if (pp->dmstate == PSTATE_FAILED) {
 				dm_reinstate(mpp->alias, pp->dev_t);
 			}
@@ -700,53 +774,34 @@ reinstate_paths (struct multipath * mpp)
 }
 
 static int
-domap (struct multipath * mpp, int action)
+domap (struct multipath * mpp)
 {
 	int op;
 	int r = 0;
-	char * act;
-
-	switch (action) {
-	case ACT_RELOAD:
-		op = DM_DEVICE_RELOAD;
-		act = ACT_RELOAD_STR;
-		break;
-
-	case ACT_CREATE:
-		op = DM_DEVICE_CREATE;
-		act = ACT_CREATE_STR;
-		break;
-
-	case ACT_SWITCHPG:
-		act = ACT_SWITCHPG_STR;
-		break;
-
-	case ACT_NOTHING:
-		act = ACT_NOTHING_STR;
-		break;
-
-	default:
-		break;
-	}
-
-	if (conf->verbosity > 1)
-		printf("%s: ", act);
 
 	print_mp(mpp);
 
 	/*
 	 * last chance to quit before touching the devmaps
 	 */
-	if (conf->dry_run)
+	if (conf->dry_run || mpp->action == ACT_NOTHING)
 		return 0;
 
-	if (action == ACT_NOTHING)
-		return 0;
-
-	if (action == ACT_SWITCHPG) {
+	if (mpp->action == ACT_SWITCHPG) {
 		dm_switchgroup(mpp->alias, mpp->nextpg);
+		/*
+		 * we may have avoided reinstating paths because there where in
+		 * active or disabled PG. Now that the topology has changed, retry.
+		 */
+		reinstate_paths(mpp);
 		return 0;
 	}
+	if (mpp->action == ACT_CREATE)
+		op = DM_DEVICE_CREATE;
+
+	if (mpp->action == ACT_RELOAD)
+		op = DM_DEVICE_RELOAD;
+
 		
 	/*
 	 * device mapper creation or updating
@@ -921,7 +976,7 @@ static void
 usage (char * progname)
 {
 	fprintf (stderr, VERSION_STRING);
-	fprintf (stderr, "Usage: %s\t[-v level] [-d] [-D major:minor] [-S]\n",
+	fprintf (stderr, "Usage: %s\t[-v level] [-d] [-S]\n",
 		progname);
 	fprintf (stderr,
 		"\t\t\t[-p failover|multibus|group_by_serial|group_by_prio]\n" \
@@ -932,8 +987,6 @@ usage (char * progname)
 		"\t   1\t\t\tprint created devmap names only\n" \
 		"\t   2\t\t\tprint all paths and multipaths\n" \
 		"\t-d\t\tdry run, do not create or update devmaps\n" \
-		"\t-D maj:min\tlimit scope to the device's multipath\n" \
-		"\t\t\t(major:minor device reference)\n"
 		"\t-S\t\tinhibit signal sending to multipathd\n"
 		"\t-F\t\tflush all multipath device maps\n" \
 		"\t-p policy\tforce all maps to specified policy :\n" \
@@ -944,7 +997,7 @@ usage (char * progname)
 		"\n" \
 		"\tdevice\t\tlimit scope to the device's multipath\n" \
 		"\t\t\t(udev-style $DEVNAME reference, eg /dev/sdb\n" \
-		"\t\t\tor a device map name)\n" \
+		"\t\t\tor major:minor or a device map name)\n" \
 		);
 
 	exit(1);
@@ -1022,7 +1075,6 @@ main (int argc, char *argv[])
 	conf->pgpolicy_flag = 0;	/* do not override defaults */
 	conf->signal = 1;		/* 1 == Send a signal to multipathd */
 	conf->dev = NULL;
-	conf->devt = NULL;
 	conf->default_selector = NULL;
 	conf->default_selector_args = 0;
 	conf->default_pgpolicy = 0;
@@ -1032,7 +1084,7 @@ main (int argc, char *argv[])
 	conf->default_features = NULL;
 	conf->default_hwhandler = NULL;
 
-	while ((arg = getopt(argc, argv, ":qdlFSi:v:p:D:")) != EOF ) {
+	while ((arg = getopt(argc, argv, ":qdlFSi:v:p:")) != EOF ) {
 		switch(arg) {
 		case 1: printf("optarg : %s\n",optarg);
 			break;
@@ -1075,10 +1127,6 @@ main (int argc, char *argv[])
 				usage(argv[0]);
 			}                
 			break;
-		case 'D':
-			conf->devt = zalloc(DEV_T_SIZE);
-			strncpy(conf->devt, optarg, DEV_T_SIZE);
-			break;
 		case ':':
 			fprintf(stderr, "Missing option arguement\n");
 			usage(argv[0]);        
@@ -1159,7 +1207,9 @@ main (int argc, char *argv[])
 		dbg("status = %s", mpp->status);
 		disassemble_map(pathvec, mpp->params, mpp);
 		disassemble_status(mpp->status, mpp);
-		reinstate_paths(mpp);
+
+		if (!conf->dry_run)
+			reinstate_paths(mpp);
 	}
 	if (conf->list) {
 		print_all_mp(curmp);
@@ -1178,14 +1228,20 @@ main (int argc, char *argv[])
 		    0 == strncmp(mpp->alias, conf->dev, FILE_NAME_SIZE)) ||
 		    0 == strncmp(mpp->wwid, conf->dev, FILE_NAME_SIZE))) {
 			setup_map(mpp);
-			domap(mpp, select_action(mpp, curmp));
+
+			if (!mpp->action)
+				select_action(mpp, curmp);
+			domap(mpp);
 			goto out;
 		}
 	}
 
 	vector_foreach_slot (mp, mpp, k) {
 		setup_map(mpp);
-		domap(mpp, select_action(mpp, curmp));
+
+		if (!mpp->action)
+			select_action(mpp, curmp);
+		domap(mpp);
 	}
 
 #if DEBUG
@@ -1201,12 +1257,5 @@ out:
 	if (conf->signal)
 		signal_daemon();
 	
-	/*
-	 * free allocs : FIXME, need a freemp() or nothing
-	 */
-	free(mp);
-	free(curmp);
-	free(pathvec);
-
 	exit(0);
 }
