@@ -55,6 +55,7 @@
 #define ACT_CREATE_STR		"create"
 
 enum actions {
+	ACT_RESERVED,
 	ACT_NOTHING,
 	ACT_RELOAD,
 	ACT_SWITCHPG,
@@ -108,6 +109,7 @@ struct path {
 	char product_id[SCSI_PRODUCT_SIZE];
 	char rev[SCSI_REV_SIZE];
 	char serial[SERIAL_SIZE];
+	unsigned long size;
 	int state;
 	int dmstate;
 	int failcount;
@@ -116,6 +118,8 @@ struct path {
 	char * getuid;
 	char * getprio;
 	int (*checkfn) (char *, char *, void **);
+
+	struct multipath * mpp;
 };
 
 struct multipath {
@@ -124,6 +128,7 @@ struct multipath {
 	int pgpolicy;
 	int nextpg;
 	int queuedio;
+	int action;
 	unsigned long size;
 	vector paths;
 	vector pg;
@@ -139,7 +144,7 @@ struct multipath {
 };
 
 struct pathgroup {
-	int id;
+	long id;
 	int status;
 	int priority;
 	vector paths;
