@@ -50,6 +50,7 @@
 #define TARGETTYPESIZE 16
 #define PARAMSSIZE 2048
 #define DEV_T_SIZE 32
+#define CMDSIZE 160
 
 #define PIDFILE "/var/run/multipathd.pid"
 #define CONFIGFILE "/etc/multipath.conf"
@@ -263,28 +264,31 @@ out:
 }
 
 static int
-updatepaths (struct paths *allpaths)
+updatepaths (struct paths *allpaths, char *sysfs_path)
 {
 	int i;
 	struct path *pp;
 	struct sysfs_directory * sdir;
 	struct sysfs_directory * devp;
 	char path[FILENAMESIZE];
-	char sysfs_path[FILENAMESIZE];
 	char attr_path[FILENAMESIZE];
 	char attr_buff[17];
 	
-	if (sysfs_get_mnt_path(sysfs_path, FILENAMESIZE)) {
-		syslog(LOG_ERR, "can not find sysfs mount point");
-		return 1;
-	}
-
 	if (safe_sprintf(path, "%s/block", sysfs_path)) {
 		fprintf(stderr, "updatepaths: path too small\n");
 		return 1;
 	}
 	sdir = sysfs_open_directory(path);
-	sysfs_read_dir_subdirs(sdir);
+
+	if (!sdir) {
+		syslog(LOG_ERR, "cannot open %s/block", sysfs_path);
+		return 1;
+	}
+	if (sysfs_read_dir_subdirs(sdir) < 0) {
+		syslog(LOG_ERR, "cannot open %s/block subdirs", sysfs_path);
+		sysfs_close_directory(sdir);
+		return 1;
+	}
 	pthread_mutex_lock(allpaths->lock);
 
 	dlist_for_each_data(sdir->subdirs, devp, struct sysfs_directory) {
@@ -301,7 +305,7 @@ updatepaths (struct paths *allpaths)
 			continue;
 		}
 		if (0 > sysfs_read_attribute_value(attr_path, attr_buff, 17)) {
-			syslog(LOG_DEBUG, "no such attribute : %s",
+			syslog(LOG_ERR, "no such attribute : %s",
 				attr_path);
 			continue;
 		}
@@ -311,8 +315,8 @@ updatepaths (struct paths *allpaths)
 		 * state persistance
 		 */
 		vector_foreach_slot (allpaths->pathvec, pp, i)
-			if (0 == strncmp(pp->dev_t, attr_buff,
-					strlen(pp->dev_t)))
+			if (!strncmp(pp->dev_t, attr_buff,
+					strlen(pp->dev_t) + 1))
 				break;
 
 		if (i < VECTOR_SIZE(allpaths->pathvec)) {
@@ -383,17 +387,21 @@ waitevent (void * et)
 {
 	struct event_thread *waiter;
 	char buff[1];
-	char * bin;
+	char cmd[CMDSIZE];
+	struct dm_task *dmt;
 
 	mlockall(MCL_CURRENT | MCL_FUTURE);
 	waiter = (struct event_thread *)et;
+
+	if (safe_snprintf(cmd, CMDSIZE, "%s %s", multipath, waiter->mapname)) {
+		syslog(LOG_ERR, "command too long, abord reconfigure");
+		return NULL;
+	}
 	pthread_mutex_lock (waiter->waiter_lock);
 
 	waiter->event_nr = geteventnr (waiter->mapname);
 	syslog(LOG_DEBUG, "waiter->event_nr = %i", waiter->event_nr);
 
-	struct dm_task *dmt;
-
 	if (!(dmt = dm_task_create(DM_DEVICE_WAITEVENT)))
 		return 0;
 
@@ -408,14 +416,9 @@ waitevent (void * et)
 out:
 	dm_task_destroy(dmt);
 
-	bin = MALLOC(strlen(multipath) + WWID_SIZE + 1);
-	sprintf(bin, "%s %s", multipath, waiter->mapname);
-
-	syslog(LOG_DEBUG, "%s", bin);
+	syslog(LOG_DEBUG, "%s", cmd);
 	syslog(LOG_NOTICE, "devmap event on %s", waiter->mapname);
-	syslog(LOG_INFO, "%s : reconfigure multipath map",
-			waiter->mapname);
-	execute_program(multipath, buff, 1);
+	execute_program(cmd, buff, 1);
 
 	/*
 	 * tell waiterloop we have an event
@@ -445,10 +448,16 @@ waiterloop (void *ap)
 	int r;
 	char buff[1];
 	int i, j;
+	char sysfs_path[FILENAMESIZE];
 
 	syslog(LOG_NOTICE, "start DM events thread");
 	mlockall(MCL_CURRENT | MCL_FUTURE);
 
+	if (sysfs_get_mnt_path(sysfs_path, FILENAMESIZE)) {
+		syslog(LOG_ERR, "can not find sysfs mount point");
+		return NULL;
+	}
+
 	/*
 	 * inits
 	 */
@@ -472,7 +481,7 @@ waiterloop (void *ap)
 			/*
 			 * we're not allowed to fail here
 			 */
-			syslog(LOG_ERR, "can't get devmaps ... retry later");
+			syslog(LOG_ERR, "can't get devmaps ... retry");
 			sleep(5);
 		}
 
@@ -480,7 +489,11 @@ waiterloop (void *ap)
 		 * update paths list
 		 */
 		syslog(LOG_INFO, "refresh failpaths list");
-		updatepaths(allpaths);
+
+		while(updatepaths(allpaths, sysfs_path)) {
+			syslog(LOG_ERR, "can't update path list ... retry");
+			sleep(5);
+		}
 
 		/*
 		 * start waiters on all devmaps
@@ -501,7 +514,7 @@ waiterloop (void *ap)
 			 */
 			if (j == VECTOR_SIZE(waiters)) {
 				wp = MALLOC (sizeof (struct event_thread));
-				strcpy (wp->mapname, devmap);
+				strncpy (wp->mapname, devmap, MAPNAMESIZE);
 				wp->thread = MALLOC (sizeof (pthread_t));
 				wp->waiter_lock = (pthread_mutex_t *) 
 					MALLOC (sizeof (pthread_mutex_t));
@@ -525,7 +538,8 @@ waiterloop (void *ap)
 				pthread_mutex_unlock (wp->waiter_lock);
 			}
 			
-			syslog(LOG_NOTICE, "event checker startup : %s", wp->mapname);
+			syslog(LOG_NOTICE, "event checker startup : %s",
+					wp->mapname);
 			pthread_create (wp->thread, &attr, waitevent, wp);
 			pthread_detach (*wp->thread);
 		}
@@ -548,7 +562,7 @@ checkerloop (void *ap)
 	int i;
 	int newstate;
 	char buff[1];
-	char * bin;
+	char cmd[CMDSIZE];
 	char checker_msg[MAX_CHECKER_MSG_SIZE];
 
 	memset(checker_msg, 0, MAX_CHECKER_MSG_SIZE);
@@ -563,6 +577,10 @@ checkerloop (void *ap)
 		syslog(LOG_DEBUG, "checking paths");
 
 		vector_foreach_slot (allpaths->pathvec, pp, i) {
+			if (!pp->checkfn) {
+				syslog(LOG_ERR, "checkfn is void");
+				continue;
+			}
 			newstate = pp->checkfn(pp->dev_t, checker_msg,
 					       &pp->checker_context);
 			
@@ -582,12 +600,17 @@ checkerloop (void *ap)
 				/*
 				 * reconfigure map now
 				 */
-				bin = MALLOC(strlen(multipath) + DEV_T_SIZE);
-				sprintf(bin, "%s -D %s", multipath, pp->dev_t);
-
-				syslog(LOG_DEBUG, "%s", bin);
-				syslog(LOG_INFO, "reconfigure multipath map");
-				execute_program(bin, buff, 1);
+				if (safe_snprintf(cmd, CMDSIZE, "%s %s",
+						  multipath, pp->dev_t)) {
+					syslog(LOG_ERR, "command too long,"
+							" abord reconfigure");
+				} else {
+					syslog(LOG_DEBUG, "%s", cmd);
+					syslog(LOG_INFO,
+						"reconfigure %s multipath",
+						pp->dev_t);
+					execute_program(cmd, buff, 1);
+				}
 
 				/*
 				 * tell waiterloop we have an event
@@ -601,7 +624,6 @@ checkerloop (void *ap)
 		pthread_mutex_unlock(allpaths->lock);
 		sleep(checkint);
 	}
-
 	return (NULL);
 }
 
